package com.xy.service;

import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.collections.map.JMap;
import com.xy.dto.DeviceQualityDto;
import com.xy.dto.DeviceQualityValueDto;
import com.xy.entity.DeviceInfo;
import com.xy.entity.DeviceQuality;
import com.xy.entity.DeviceQualityValue;
import com.xy.entity.SysDictRedis;
import com.xy.error.CommRuntimeException;
import com.xy.mapper.DeviceInfoMapper;
import com.xy.mapper.DeviceQualityMapper;
import com.xy.utils.*;
import com.xy.utils.enums.DictEnum;
import com.xy.utils.enums.DictSonEnum;
import com.xy.utils.enums.QualityEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static com.xy.utils.PlusBeans.*;

/**
 * <p>
 * 设备-质检 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-05
 */
@Service
@AllArgsConstructor
@Api(tags = "设备-质检")
public class DeviceQualityServiceImpl extends ServiceImpl<DeviceQualityMapper, DeviceQuality> implements DeviceQualityService {

    private DeviceInfoMapper deviceInfoMapper;

    private DeviceQualityValueServiceImpl deviceQualityValueService;

    private void queryDeviceQualityValue(List<DeviceQualityDto.Vo> list) {
        if (Emptys.check(list)) {
            JList<Integer> qualityIds = new JArrayList<>(list).getProperty(DeviceQualityDto.Vo::getQualityId);
            List<DeviceQualityValue> deviceQualityValues = deviceQualityValueService.list(new LambdaQueryWrapper<DeviceQualityValue>().in(DeviceQualityValue::getQualityId, qualityIds));
            JMap<Integer, DeviceQualityValue> deviceQualityValueMap = new JArrayList<>(deviceQualityValues).toMap(DeviceQualityValue::getQualityId).cover();
            list.forEach(vo -> {
                DeviceQualityValue deviceQualityValue = deviceQualityValueMap.get(vo.getQualityId());
                if (deviceQualityValue == null) {
                    return;
                }
                vo.setDeviceQualityValue(copy(DeviceQualityValueDto.Vo.class, deviceQualityValue));
            });
        }
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<DeviceQualityDto.Vo>> page(@RequestBody DeviceQualityDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<DeviceQuality> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, DeviceQuality.class)
                .ge(DeviceQuality::getUpdateTime, page.getBeginUpdateTime())
                .le(DeviceQuality::getUpdateTime, page.getEndUpdateTime())
                .build()
                .orderByDesc(!Emptys.check(pageBean.getOrders()), DeviceQuality::getCreateTime);
        IPage<DeviceQuality> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        PageBean<DeviceQualityDto.Vo> voPageBean = toPageBean(DeviceQualityDto.Vo.class, iPage);
        List<DeviceQualityDto.Vo> list = voPageBean.getRecords();
        queryDeviceQualityValue(list);
        return R.ok(voPageBean);
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<DeviceQualityDto.Vo>> list(@RequestBody DeviceQualityDto.SelectList selectList) {
        LambdaQueryWrapper<DeviceQuality> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceQuality.class)
                .in(DeviceQuality::getQualityId, selectList.getQualityIds())
                .in(DeviceQuality::getDeviceId, selectList.getDeviceIds())
                .build();
        List<DeviceQuality> deviceQualities = list(lambdaQueryWrapper);
        List<DeviceQualityDto.Vo> list = copy(DeviceQualityDto.Vo.class, deviceQualities);
        queryDeviceQualityValue(list);
        return R.ok(list);
    }

    @PostMapping("obj")
    @ApiOperation("对象查询")
    public R<DeviceQualityDto.Vo> obj(@RequestBody DeviceQualityDto.Obj obj) {
        LambdaQueryWrapper<DeviceQuality> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(obj, DeviceQuality.class).build();
        List<DeviceQuality> deviceQualities = list(lambdaQueryWrapper);
        List<DeviceQualityDto.Vo> list = copy(DeviceQualityDto.Vo.class, deviceQualities);
        if (!Emptys.check(list)) {
            if (Emptys.check(obj.getDeviceId())) {
                save(Arrays.asList(new DeviceQualityDto.Save().setDeviceId(obj.getDeviceId())));
                return obj(obj);
            }
            return R.ok();
        }
        DeviceQualityDto.Vo vo = copy(DeviceQualityDto.Vo.class, list.get(0));
        queryDeviceQualityValue(Arrays.asList(vo));
        return R.ok(vo);
    }

    @PostMapping("update")
    @ApiOperation("修改")
    public R update(@RequestBody List<DeviceQualityDto.Vo> vos) {
        List<DeviceQuality> deviceQualities = copy(DeviceQuality.class, vos);
        LocalDateTime now = LocalDateTime.now();
        for (DeviceQuality deviceQuality : deviceQualities) {
            if (deviceQuality.getQualityState() != null) {
                deviceQuality.setQualityAuditUserId(AuthorizeUtils.getLoginId(Long.class));
            }
            if (deviceQuality.getFactoryState() != null) {
                deviceQuality.setFactoryAuditUserId(AuthorizeUtils.getLoginId(Long.class));
            }
            deviceQuality.setUpdateTime(now);
        }
        updateBatchById(deviceQualities);
        return R.ok();
    }

    @PostMapping("rest")
    @ApiOperation(value = "重置多项质检", hidden = true)
    public R rest(@RequestBody DeviceQualityDto.Rest rest) {
        SysDictRedis sysDictRedis1 = SysDictUtils.get(DictEnum.DEVICE_QUALITY_STATUS.getKey(), DictSonEnum.DEVICE_QUALITY_STATUS_QUALIFIED.getKey());
        SysDictRedis sysDictRedis2 = SysDictUtils.get(DictEnum.DEVICE_FACTORY_STATUS.getKey(), DictSonEnum.DEVICE_FACTORY_STATUS_CAN_SHIPPED.getKey());
        DeviceQuality deviceQuality = getById(rest.getQualityId());
        if (sysDictRedis1.getValue().equals(deviceQuality.getQualityState()) || sysDictRedis2.getValue().equals(deviceQuality.getFactoryState())) {
            return R.fail("设备已合格或已出厂");
        }
        List<String> types = rest.getTypes();
        LocalDateTime now = LocalDateTime.now();
        types.forEach(type -> {
            QualityEnum qualityEnum = QualityEnum.getQualityEnum(type);
            if (qualityEnum != null) {
                //修改质检
                UpdateWrapper<DeviceQuality> updateWrapper = new UpdateWrapper<DeviceQuality>()
                        .set(qualityEnum.getBeginTime(), now)
                        .set(qualityEnum.getEndTime(), null)
                        .set(StringTools.humpToLine(LambdaUtils.getProperty(DeviceQuality::getUpdateTime)), now)
                        .eq(StringTools.humpToLine(LambdaUtils.getProperty(DeviceQuality::getQualityId)), rest.getQualityId());
                update(updateWrapper);
                //删除质检属性
                DeviceQualityValueDto.Vo del = new DeviceQualityValueDto.Vo()
                        .setQualityId(rest.getQualityId())
                        .setQualityType(type);
                deviceQualityValueService.del(del);
            }
        });
        return R.ok();
    }

    @PostMapping("save")
    @ApiOperation("添加新质检")
    public R<List<Tuple.Tuple3<Long, Long, Boolean>>> save(@RequestBody List<DeviceQualityDto.Save> deviceQualitySaves) {
        List<DeviceQualityDto.Vo> updates = new ArrayList<>();
        List<DeviceQuality> saves = new ArrayList<>();
        JList<Long> deviceIds = new JArrayList<>(deviceQualitySaves).getProperty(DeviceQualityDto.Save::getDeviceId);
        //获取设备信息
        List<DeviceInfo> deviceInfos = deviceInfoMapper.selectList(new LambdaQueryWrapper<DeviceInfo>().in(DeviceInfo::getDeviceId, deviceIds));
        JMap<Long, DeviceInfo> deviceInfoJMap = new JArrayList<>(deviceInfos).toMap(DeviceInfo::getDeviceId).cover();
        //获取已有质检信息
        List<DeviceQuality> deviceQualities = list(new LambdaQueryWrapper<DeviceQuality>()
                .in(DeviceQuality::getDeviceId, deviceIds)
                .eq(DeviceQuality::getShowStatus, true)
        );
        JMap<Long, List<DeviceQuality>> deviceQualitieJMap = new JArrayList<>(deviceQualities).toMap(DeviceQuality::getDeviceId).group();
        LocalDateTime now = LocalDateTime.now();
        //封装修改和添加对象
        deviceQualitySaves.forEach(save -> {
            DeviceInfo deviceInfo = deviceInfoJMap.get(save.getDeviceId());
            if (deviceInfo == null) {
                throw new CommRuntimeException(save.getDeviceId() + "设备不存在");
            }
            DeviceQuality deviceQuality = new DeviceQuality()
                    .setDeviceId(save.getDeviceId())
                    .setDeviceType(deviceInfo.getDeviceType())
                    .setCreateTime(now);
            List<DeviceQuality> list = deviceQualitieJMap.get(deviceQuality.getDeviceId());
            if (Emptys.check(list)) {
                JList<Integer> qualityIds = new JArrayList<>(list).getProperty(DeviceQuality::getQualityId);
                List<DeviceQualityDto.Vo> vos2 = new ArrayList<>(qualityIds.size());
                qualityIds.forEach(qualityId -> vos2.add(new DeviceQualityDto.Vo().setQualityId(qualityId).setShowStatus(false)));
                updates.addAll(vos2);
            }
            saves.add(deviceQuality);
        });
        //修改
        if (Emptys.check(updates)) {
            update(updates);
        }
        //添加
        saveBatch(saves);
        return R.ok();
    }

    @ApiOperation(value = "质检结果通知", hidden = true)
    @PostMapping("qualityResultBack")
    public R qualityResultBack(@RequestBody DeviceQualityDto.QualityResultBack qualityResultBack) {
        Long deviceId = qualityResultBack.getDeviceId();
        String type = qualityResultBack.getType();
        QualityEnum qualityEnum = QualityEnum.getQualityEnum(type);
        if (qualityEnum == null) {
            return R.ok();
        }
        JSONObject data = qualityResultBack.getData();
        LambdaQueryWrapper<DeviceQuality> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceQuality>()
                .eq(DeviceQuality::getDeviceId, deviceId)
                .eq(DeviceQuality::getShowStatus, true);
        DeviceQuality deviceQuality = getOne(lambdaQueryWrapper);
        if (deviceQuality == null) {
            return R.fail("设备不存在");
        }
        //修改质检
        UpdateWrapper<DeviceQuality> updateWrapper = new UpdateWrapper<DeviceQuality>()
                .set(qualityEnum.getEndTime(), LocalDateTime.now())
                .eq(StringTools.humpToLine(LambdaUtils.getProperty(DeviceQuality::getDeviceId)), deviceId)
                .eq(StringTools.humpToLine(LambdaUtils.getProperty(DeviceQuality::getShowStatus)), true);
        update(updateWrapper);
        //修改质检值
        DeviceQualityValueDto.Vo saveOrUpdate = new DeviceQualityValueDto.Vo()
                .setQualityType(type)
                .setQualityValue(data.toString());
        saveOrUpdate.setQualityId(deviceQuality.getQualityId());
        deviceQualityValueService.saveOrUpdate(saveOrUpdate);
        return R.ok();
    }

    @PostMapping("getQrCode")
    @ApiOperation("获取二维码")
    public R<String> getQrCode(@RequestBody DeviceQualityDto.Qrcode qrcode) {
        SysDictRedis sysDictRedis = SysDictUtils.get(DictEnum.DEVICE_QR_CODE.getKey(), DictSonEnum.DEVICE_QR_CODE_URL.getKey());
        String deviceId = qrcode.getDeviceId().toString();
        String base64 = QRCodeUtils.create(sysDictRedis.getValue() + deviceId, 400, 400, "back1.png", deviceId).base64();
        return R.ok(base64);
    }

    @PostMapping("exportQrCode")
    @ApiOperation("导出二维码")
    public void exportQrCode(@RequestBody DeviceQualityDto.Qrcode qrcode) {
        SysDictRedis sysDictRedis = SysDictUtils.get(DictEnum.DEVICE_QR_CODE.getKey(), DictSonEnum.DEVICE_QR_CODE_URL.getKey());
        String deviceId = qrcode.getDeviceId().toString();
        QRCodeUtils.create(sysDictRedis.getValue() + deviceId, 400, 400, "back1.png", deviceId).download(deviceId);
    }

}
